####################################################################
#																   	
# Geant2 End-2-End-Monitoring System							   	
# 																   	
# $Source: /cnm/cvs-root//G2_E2E/G2_E2E_MP/src/G2_E2E_MP.pm,v $
# $Revision: 1.5 $
# $Date: 2006/09/27 07:11:13 $												
# $Author: a2824ch $
#	
# Description: 	Simple to use E2E Measurement Point for perfSONAR												
# 
# Authors:		M&M Team, LRZ Muenchen 
#                   Matthias Hamm   ~ hamm@dfn.de
#                   Mark Yampolskiy ~ myy@dfn.de
#
# more Info: 	http://wiki.geant2.net/bin/view/JRA4/Jra4WI3 		
#
####################################################################

use SOAP::Lite; 
package G2_E2E_MP;

use vars qw(@ISA);
@ISA=qw(SOAP::Server::Parameters);

use utf8;

use XML::DOM;

## for prototype only, the expected request file is added inline as a "here-file"
## for release version, the request has to be parsed properly, of course ;-)
## NOTE: The XML header and SOAP Envelope is included in the raw XML message, so we have to add it here
my $req = <<reqfile
<?xml version="1.0" encoding="UTF-8"?><soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"><soap:Body><MPService xmlns="G2_E2E_MP"><nmwg:message type="SetupDataRequest"
               xmlns:nmwg="http://ggf.org/ns/nmwg/base/2.0/">

     <nmwg:metadata id="meta1">
         <nmwg:eventType>Path.Status</nmwg:eventType>
     </nmwg:metadata>

     <nmwg:data id="data1" metadataIdRef="meta1"/>
    </nmwg:message>
</MPService></soap:Body></soap:Envelope>
reqfile
;

no utf8;

sub ReadFileToString
{
	my $file = shift;
	my $fstr = "";
	
	# Read file in to string
	open REQ, "<".$file or die "Could not open $file\n";
	foreach my $line (<REQ>) {
	  $fstr .= $line unless $line =~ m/^<\?xml.*/;
	}
	close REQ;
	
	return $fstr;
}

sub WriteStringToFile
{
	my $file = shift;
	my $str = shift;
	
	# Write string in to the file
	open REQ, ">".$file or die "Could not open $file\n";
	print REQ $str;
	close REQ;

}

sub MPService {
  shift;				# remove class name
  my $xmlmsg = shift;
  my $som = shift;
  my $rawxml_msg = $som->{_raw_xml};
  my $errMsg;

  
  # NOTE: For the prototype version, a very simplified check of the request is done
  # For release version, it has to be replaced with a proper logical check of the XML request

 
  WriteStringToFile("traceRequest.txt","req:\n".$req."\n========\nrawxml_msg:\n".$rawxml_msg);
  
  my $parser = new XML::DOM::Parser;
  my $doc = $parser->parse ($rawxml_msg);
 
  # get root element (skip SOAP envelope)
  my $eventType = ($doc->getElementsByTagName ("nmwg:eventType"))[0];
  if ($eventType==undef)
  {
      $errMsg = "ERROR: bad request, nmwg:eventType missing (".$rawxml_msg.")\n";
      goto exit_error;
  }
  
  $event = $eventType->getFirstChild->getData();
  if ($event ne "Path.Status")
  {
      $errMsg = "ERROR: bad type of request, eventType ".$event." not understood by this MP (".$rawxml_msg.")\n";
      goto exit_error;
  }

  # read reg. file to string
  my $reg_file = ReadFileToString ($::XML_response);
  
  # replace "<nmwg:store>" through "<nmwg:message>" (required for perfSONAR schema)
  $reg_file =~ s/nmwg:store/nmwg:message/g;


  $doc->dispose;    # release DOM objects (does not work 100% properly)
  return $reg_file;

exit_error:

  $doc->dispose;
  return $errMsg; 
}






1;

