#!perl -w +trace

####################################################################
#																   	
# Geant2 End-2-End-Monitoring System							   	
# 																   	
# $Source: /cnm/cvs-root//G2_E2E/G2_E2E_MP/src/G2_E2E_ServerDaemon.pl,v $
# $Revision: 1.5 $
# $Date: 2006/07/28 07:08:18 $												
# $Author: a2824cn $
#	
# Description: 	SOAP Daemon for E2E Measurement Point												
# 
# Authors:		M&M Team, LRZ Muenchen 
#                   Matthias Hamm   ~ hamm@dfn.de
#                   Mark Yampolskiy ~ myy@dfn.de
#
# more Info: 	http://wiki.geant2.net/bin/view/JRA4/Jra4WI3 		
#
####################################################################
use SOAP::Lite;
#use SOAP::Lite +trace => [qw(debug)];
use SOAP::Transport::HTTP;
#use SOAP::Transport::HTTP::Daemon;
use SOAPTransportHTTP_Patch;

# for comand line parameter processing
use Getopt::Long qw(:config bundling);
use Pod::Usage;

# Parse command line parameters
my $host = undef;
my $port = undef;
$XML_response = undef;

my $help = 0;

GetOptions(
  "help|?" => \$help,
  "host|h=s"        => \$host,
  "port|p=s"        => \$port,
  "response|r=s"    => \$XML_response
) or pod2usage(2);

pod2usage(1) if $help;

pod2usage("Mandatory parameter missing.\n")
  unless defined $host && defined $port;
  
my $daemon;

$SIG{TERM} = sub { print STDERR "closing socket\n"; close $daemon->close; };
$SIG{INT} = sub { print STDERR "closing socket\n"; close $daemon->close; };
 
# MKH 17/07/06 use patch for access to raw xml
$daemon = SOAPTransportHTTP_Patch::Daemon 
#$daemon = SOAP::Transport::HTTP::Daemon
  ->new( LocalAddr => $host, LocalPort => $port )
  ->dispatch_to('G2_E2E_MP::MPService')
  # enable compression support
  ->options( { compress_threshold => 10000 } );
#bless($daemon,"SOAPTransportHTTP_Patch");
print "Geant2 E2E Monitoring System - SOAP Server Side started. Contact at ", $daemon->url, "\n";
$daemon->handle;

## END OF MAIN ##


__END__



=head1 NAME

B<G2_E2E_ServerDaemon.pl> - Handle perfSONAR requests for Geant2 E2E Monitoring Data

=head1 SYNOPSIS

B<G2_E2E_ServerDaemon.pl> S<[B<--help>]> 
  S<B<--host>|B<-h>=HOSTNAME> S<B<--port>|B<-p>=PORT>>

=head1 DESCRIPTION

Handle perfSONAR requests for Geant2 E2E Monitoring Data.

=head1 OPTIONS

=over

=item B<--help>

Prints a help message and exits.

=item B<--host>|B<-h>=HOST

Register SOAP server on host HOST. Mandatory parameter!

=item B<--port>|B<-p>=PORT

Register SOAP server on port PORT. Mandatory parameter!

=head1 EXAMPLES

S<G2_E2E_ServerDaemon.pl> S<--host=localhost> S<--port=8080> 



