####################################################################
#																   	
# Geant2 End-2-End-Monitoring System							   	
# 																   	
# $Source$
# $Revision$
# $Date$												
# $Author$ 
#	
# Description: 	Patch for SOAP Lite. Adds access to raw XML	(via new hash value _raw_xml of SOM)
#               Note: This patch is based on SOAP::Lite 0.66.1											
# 
# Authors:		M&M Team, LRZ Muenchen 
#                   Matthias Hamm   ~ hamm@dfn.de
#                   Mark Yampolskiy ~ myy@dfn.de
#
# more Info: 	http://wiki.geant2.net/bin/view/JRA4/Jra4WI3 		
#
####################################################################

package SOAPServerPatch;

use vars qw(@ISA);
@ISA=qw(SOAP::Server);


sub handle {
        print ("* SOAP_Communication::SOAPServerPatch->handle * \n");
  SOAP::Trace::trace('()');
  my $self = shift;
  $self = $self->new if !ref $self; # inits the server when called in a static context
  $self->init_context();
  # we want to restore it when we are done
  local $SOAP::Constants::DEFAULT_XML_SCHEMA
    = $SOAP::Constants::DEFAULT_XML_SCHEMA;

  # SOAP version WILL NOT be restored when we are done.
  # is it problem?

  my $result = eval {
    local $SIG{__DIE__};
    # why is this here:
    $self->serializer->soapversion(1.1);
    my $request = eval { $self->deserializer->deserialize($_[0]) };

# MKH 17/07/06 - added to retrieve raw xml input to server
#                thanks to DFN Erlangen
  $request->{_raw_xml} = $_[0];

    die SOAP::Fault
      ->faultcode($SOAP::Constants::FAULT_VERSION_MISMATCH)
        ->faultstring($@)
	  if $@ && $@ =~ /^$SOAP::Constants::WRONG_VERSION/;
    die "Application failed during request deserialization: $@" if $@;
    my $som = ref $request;
    die "Can't find root element in the message" 
      unless $request->match($som->envelope);
    $self->serializer->soapversion(SOAP::Lite->soapversion);
    $self->serializer->xmlschema($SOAP::Constants::DEFAULT_XML_SCHEMA
				 = $self->deserializer->xmlschema)
      if $self->deserializer->xmlschema;

    die SOAP::Fault
      ->faultcode($SOAP::Constants::FAULT_MUST_UNDERSTAND)
	->faultstring("Unrecognized header has mustUnderstand attribute set to 'true'")
	  if !$SOAP::Constants::DO_NOT_CHECK_MUSTUNDERSTAND &&
	    grep { $_->mustUnderstand && 
		     (!$_->actor || $_->actor eq $SOAP::Constants::NEXT_ACTOR)
		   } $request->dataof($som->headers);

    die "Can't find method element in the message"
      unless $request->match($som->method);
    # TODO - SOAP::Dispatcher plugs in here
    # my $handler = $self->dispatcher->find_handler($request);
    my($class, $method_uri, $method_name) = $self->find_target($request);
    my @results = eval {
      local $^W;
      my @parameters = $request->paramsin;

      # SOAP::Trace::dispatch($fullname);
      SOAP::Trace::parameters(@parameters);

      push @parameters, $request 
        if UNIVERSAL::isa($class => 'SOAP::Server::Parameters');

      SOAP::Server::Object->references(
	  defined $parameters[0] && ref $parameters[0] &&
          UNIVERSAL::isa($parameters[0] => $class) ? do {
	    my $object = shift @parameters;
	    SOAP::Server::Object->object(ref $class ? $class : $object)
		->$method_name(SOAP::Server::Object->objects(@parameters)),
		  # send object back as a header
		  # preserve name, specify URI
		  SOAP::Header
		      ->uri($SOAP::Constants::NS_SL_HEADER => $object)
			->name($request->dataof($som->method.'/[1]')->name)
		      } # end do block
	 # SOAP::Dispatcher will plug-in here as well
         # $handler->dispatch(SOAP::Server::Object->objects(@parameters)
       : $class->$method_name(SOAP::Server::Object->objects(@parameters)) );
    }; # end eval block
    SOAP::Trace::result(@results);

    # let application errors pass through with 'Server' code
    die ref $@ ?
      $@ : $@ =~ /^Can\'t locate object method "$method_name"/ ?
	"Failed to locate method ($method_name) in class ($class)" :
	  SOAP::Fault->faultcode($SOAP::Constants::FAULT_SERVER)->faultstring($@)
	      if $@;

    my $result = $self->serializer
      ->prefix('s') # distinguish generated element names between client and server
      ->uri($method_uri)
      ->envelope(response => $method_name . 'Response', @results);
    $self->destroy_context();
    return $result;
  };

  $self->destroy_context();
  # void context
  return unless defined wantarray;

  # normal result
  return $result unless $@;

  # check fails, something wrong with message
  return $self->make_fault($SOAP::Constants::FAULT_CLIENT, $@) unless ref $@;

  # died with SOAP::Fault
  return $self->make_fault($@->faultcode   || $SOAP::Constants::FAULT_SERVER,
			   $@->faultstring || 'Application error',
			   $@->faultdetail, $@->faultactor)
    if UNIVERSAL::isa($@ => 'SOAP::Fault');

  # died with complex detail
  return $self->make_fault($SOAP::Constants::FAULT_SERVER, 'Application error' => $@);

} # end of handle()

1;
