package SNMPmonConfig;
########################################
#
#  maxim@fnal.gov 2007, MG
#
#    
#   configurable parameters for SNMP monitoring
#   stored on disk as config parsable by Config::General
# 
 
use strict;
use warnings;
use Config::General;
use File::Copy; 
BEGIN {
        use Exporter   ();
         
        our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
      
        # set the version for version checking
        $VERSION     = 1.00;
        # if using RCS/CVS, this may be preferred
        $VERSION = sprintf "%d.%03d", q$Revision: 1.1 $ =~ /(\d+)/g;

        @ISA         = qw(Exporter);
        @EXPORT      = qw( );
        %EXPORT_TAGS = (  
                         Monitor => [qw!%oids %BgpPeerState %ifOperState $SNMP_CONFIG_OBJ   &storeSNMP!]);     # eg: TAG => [ qw!name1 name2! ],
                        
        # your exported package globals go here,
        # as well as any optionally exported functions
        @EXPORT_OK   = qw(  %oids %BgpPeerState %ifOperState $SNMP_CONFIG_OBJ   &storeSNMP     );                        
     };
our @EXPORT_OK;
use E2EConfig qw($debug  %GENERAL_CONFIG); 
our (  %oids, %BgpPeerState, %ifOperState,  $SNMP_CONFIG_OBJ  );
 
 # OID's to the SNMP elements that I want to show...
# From Cisco's MIB and RFC's
# http://tools.cisco.com/Support/SNMP/do/BrowseMIB.do?local=en&step=2&mibName=BGP4-MIB
%oids = ( 
        "SysUptime"                     =>      "1.3.6.1.2.1.1.3.0",
        "bgpVersion"                    =>      "1.3.6.1.2.1.15.1.0",
        "bgpLocalAs"                    =>      "1.3.6.1.2.1.15.2.0",

#       "bgpPeerTable"                  =>      "1.3.6.1.2.1.15.3",
        "bgpPeerEntry"                  =>      "1.3.6.1.2.1.15.3.1",
        "bgpPeerIdentifier"             =>      "1.3.6.1.2.1.15.3.1.1",
        "bgpPeerState"                  =>      "1.3.6.1.2.1.15.3.1.2",
        "bgpPeerAdminStatus"            =>      "1.3.6.1.2.1.15.3.1.3",
        "bgpPeerNegotiatedVersion"      =>      "1.3.6.1.2.1.15.3.1.4",
        "bgpPeerLocalAddr"              =>      "1.3.6.1.2.1.15.3.1.5",
        "bgpPeerLocalPort"              =>      "1.3.6.1.2.1.15.3.1.6",
        "bgpPeerRemoteAddr"             =>      "1.3.6.1.2.1.15.3.1.7",
        "bgpPeerRemotePort"             =>      "1.3.6.1.2.1.15.3.1.8",
        "bgpPeerRemoteAs"               =>      "1.3.6.1.2.1.15.3.1.9",
        "bgpPeerInUpdates"              =>      "1.3.6.1.2.1.15.3.1.10",
        "bgpPeerOutUpdates"             =>      "1.3.6.1.2.1.15.3.1.11",
        "bgpPeerInTotalMessages"        =>      "1.3.6.1.2.1.15.3.1.12",
        "bgpPeerOutTotalMessages"       =>      "1.3.6.1.2.1.15.3.1.13",
        "bgpPeerLastError"              =>      "1.3.6.1.2.1.15.3.1.14",
        "bgpPeerFsmEstablishedTransitions" =>   "1.3.6.1.2.1.15.3.1.15",
        "bgpPeerFsmEstablishedTime"     =>      "1.3.6.1.2.1.15.3.1.16",
        "bgpPeerConnectRetryInterval"   =>      "1.3.6.1.2.1.15.3.1.17",
        "bgpPeerHoldTime"               =>      "1.3.6.1.2.1.15.3.1.18",
        "bgpPeerKeepAlive"              =>      "1.3.6.1.2.1.15.3.1.19",
        "bgpPeerHoldTimeConfigured"     =>      "1.3.6.1.2.1.15.3.1.20",
        "bgpPeerKeepAliveConfigured"    =>      "1.3.6.1.2.1.15.3.1.21",
        "bgpPeerMinASOriginationInterval" =>    "1.3.6.1.2.1.15.3.1.22",
        "bgpPeerMinRouteAdvertisementInterval" => "1.3.6.1.2.1.15.3.1.23",
        "bgpPeerInUpdateElapsedTime"    =>      "1.3.6.1.2.1.15.3.1.24",
        "bgpIdentifier"                 =>      "1.3.6.1.2.1.15.4",
        "bgpRcvdPathAttrTable"          =>      "1.3.6.1.2.1.15.5",
        "bgp4PathAttrTable"             =>      "1.3.6.1.2.1.15.6",
        "bgpPathAttrEntry"              =>      "1.3.6.1.2.1.15.5.1",
        "bgpPathAttrPeer"               =>      "1.3.6.1.2.1.15.5.1.1",
        "bgpPathAttrDestNetwork"        =>      "1.3.6.1.2.1.15.5.1.2",
        "bgpPathAttrOrigin"             =>      "1.3.6.1.2.1.15.5.1.3",
        "bgpPathAttrASPath"             =>      "1.3.6.1.2.1.15.5.1.4",
        "bgpPathAttrNextHop"            =>      "1.3.6.1.2.1.15.5.1.5",
        "bgpPathAttrInterASMetric"      =>      "1.3.6.1.2.1.15.5.1.6",
        "bgp4PathAttrEntry"             =>      "1.3.6.1.2.1.15.6.1",
        "bgp4PathAttrPeer"              =>      "1.3.6.1.2.1.15.6.1.1",
        "bgp4PathAttrIpAddrPrefixLen"   =>      "1.3.6.1.2.1.15.6.1.2",
        "bgp4PathAttrIpAddrPrefix"      =>      "1.3.6.1.2.1.15.6.1.3",
        "bgp4PathAttrOrigin"            =>      "1.3.6.1.2.1.15.6.1.4",
        "bgp4PathAttrASPathSegment"     =>      "1.3.6.1.2.1.15.6.1.5",
        "bgp4PathAttrNextHop"           =>      "1.3.6.1.2.1.15.6.1.6",
        "bgp4PathAttrMultiExitDisc"     =>      "1.3.6.1.2.1.15.6.1.7",
        "bgp4PathAttrLocalPref"         =>      "1.3.6.1.2.1.15.6.1.8",
        "bgp4PathAttrAtomicAggregate"   =>      "1.3.6.1.2.1.15.6.1.9",
        "bgp4PathAttrAggregatorAS"      =>      "1.3.6.1.2.1.15.6.1.10",
        "bgp4PathAttrAggregatorAddr"    =>      "1.3.6.1.2.1.15.6.1.11",
        "bgp4PathAttrCalcLocalPref"     =>      "1.3.6.1.2.1.15.6.1.12",
        "bgp4PathAttrBest"              =>      "1.3.6.1.2.1.15.6.1.13",
        "bgp4PathAttrUnknown"           =>      "1.3.6.1.2.1.15.6.1.14",
	"vtpVlanState"                  =>      "1.3.6.1.4.1.9.9.46.1.3.1.1.2.1", 
	"ifOperStatus"                  =>      "1.3.6.1.2.1.2.2.1.8",
	"ifName"                        =>      "1.3.6.1.2.1.31.1.1.1.1", 
	"ip.ipAddrTable.ipAddrEntry"    =>      "1.3.6.1.2.1.4.20.1",
        );


  %BgpPeerState = (
       1 => "idle",
       2 => "connect",
       3 => "active",
       4 => "opensnet",
       5 => "openconfirm",
       6 => "established"
       );
  %ifOperState = (
        1 => "up",
        2 => "down",
        3 => "testing",
        4 => "unknown",
        5 => "dormant",
        6 => "notPresent",
	7 =>  "lowerLayerDown"
	);
 
 	
 $SNMP_CONFIG_OBJ = new Config::General(-ConfigFile => $GENERAL_CONFIG{SNMPCONFIG_FILE}, 
                                       -StoreDelimiter => '=');

 #
#    save SNMP config as hash in SNMPmonConfig.pm
#
sub storeSNMP {
   my($ssid, $mylinks) = @_;

      my $tmp_file = "$GENERAL_CONFIG{SNMPCONFIG_FILE}."  . $ssid;
      my $comments = 
qq{#############################################################################################
#   This file is edited by utilizing E2E configuration management webinterface
#   if you'd like to configure it manually then data format of is(enter data in [...]:
#   
#   <[interface IP address]>    
#                     rtr=  [router hostname] 
#		      <[interface index]> 
#		                   status  =   <[[123], where 1 - check only physical interface, 2 - add check for VLAN, 3 - add check for bgp]>
#				   bgp  =  <[bgp neighbour IP address]> 
#				   vlan  =  <[vlan index]> 
#                                  <datum>   
#                                       [mdXX  - metadata id] =   <[global E2E link name]>
#		                   </datum>
#		     </[interface index]>	             
#   </[interface IP address]>                            
#};    
     open OUTF, ">$tmp_file" or croak(" Failed to open: $tmp_file");  
     $SNMP_CONFIG_OBJ = new Config::General(-ConfigHash =>  $mylinks, 
                                       -StoreDelimiter => '='); 
     print  OUTF "$comments\n";
     print  OUTF $SNMP_CONFIG_OBJ->save_string();
     close OUTF;
     unless($debug) {
        move($tmp_file,  $GENERAL_CONFIG{SNMPCONFIG_FILE} ) or croak(" failed to move from $tmp_file to $GENERAL_CONFIG{SNMPCONFIG_FILE} ");  
     }
     
}  
                    
1;
