package XMLUtils;
########################################
#
#  maxim@fnal.gov 2007, MG
#
#    
#  module for parsing XML  files and storing it
#  
# 
 
use strict;
use warnings;
use POSIX qw(strftime);
use Carp;
use XML::Writer; 
use XML::XPath;
use IO::File;
BEGIN {
        use Exporter   ();
         
        our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
      
        # set the version for version checking
        $VERSION     = 1.00;
        # if using RCS/CVS, this may be preferred
        $VERSION = sprintf "%d.%03d", q$Revision: 1.1 $ =~ /(\d+)/g;

        @ISA         = qw(Exporter);
        @EXPORT      = qw( );
        %EXPORT_TAGS = (  
                       XMLConfig =>  [qw!&storeTemplate &fillXML &sendMARequest &storeMA &reloadSQLMA &fillConfig!]  
		       );
                        
        # your exported package globals go here,
        # as well as any optionally exported functions
        @EXPORT_OK   = qw(  &fillXML &storeTemplate &storeMA &sendMARequest &reloadSQLMA &fillConfig  );                        
     };
use E2EConfig    qw($debug  %GENERAL_CONFIG); 
our @EXPORT_OK;
$ENV{PATH} = '/usr/bin:/usr/local/bin';
 
  
#---------------------------------------------------------
#
#   xml parsing helper sub -  returns found attribute from the XML element
#
sub get_attr {
  my($attrs, $name) = @_;
  if (ref($attrs) =~ /ARRAY/) {
     foreach my $attr ( @{$attrs}) {
        return $attr->getData() if($attr->getName && $attr->getName eq $name);
     } 
  } elsif ($attrs && $attrs->getName() && $attrs->getName eq $name) {
     return $attrs->getData();
  } else {
     return undef;
  }
}
#
#    send data store request to local MA
#
sub sendMARequest {
if( -e $GENERAL_CONFIG{statusFileMA} ) {
        eval {
	 $ENV{JAVA_HOME} = $GENERAL_CONFIG{JAVA_HOME};
	 $ENV{ANT_HOME} = $GENERAL_CONFIG{ANT_HOME};
	 $ENV{CLASSPATH} = $GENERAL_CONFIG{JAVA_CLASSPATH};
         `$GENERAL_CONFIG{MA_REQUEST}`
      } ;
      if($@) {
         croak (" Failed to send store L2 E2Edata request to MA: $@");
      }
    } 

}
#
#   reload SQL MA configuraton file and redeploy DB 
#    ( configure suxec in order to run this as DB admin user or allow apache user to do that - !!! not recommended !!!)
#
sub reloadSQLMA {
 
if( -e  $GENERAL_CONFIG{sqlMa_config}) {
        eval {
	 $ENV{JAVA_HOME} = $GENERAL_CONFIG{JAVA_HOME};
	 $ENV{ANT_HOME} =  $GENERAL_CONFIG{ANT_HOME};
	 $ENV{CLASSPATH} = $GENERAL_CONFIG{JAVA_CLASSPATH};
         `$GENERAL_CONFIG{SQL_MA_RECONFIG}`
      } ;
      if($@) {
         croak (" Failed to reconfigure and redeploy SQL MA: $@");
      }
 }
}
#
#  store  SQL MA config file ( the one in the service.properties file
#   service.ma.conf_file=<filename> 
#
#
sub storeSQLMAConfig {
   my ($ssid, $nodes, $extnodes,  $config) = @_;

   my $tmp_file = "$GENERAL_CONFIG{sqlMa_config}."  . $ssid ;
   my $fd = new IO::File(">$tmp_file")  or croak("Failed to open file $tmp_file" . $!);
   
   my $namespace  = "http://ggf.org/ns/nmwg/base/2.0/";##
   my %ibatis_configs = ('ID_LinkPartialInfo'=>$GENERAL_CONFIG{sqlMa_ibatisIDConfig},
                         'DOMAIN_Link' =>$GENERAL_CONFIG{sqlMa_ibatisDomainConfig});                                        
   my $writer = new XML::Writer(NAMESPACES => 1,
                                PREFIX_MAP => { $namespace => "store",},
                                UNSAFE => 1, OUTPUT =>  $fd, DATA_MODE => 1, DATA_INDENT => 2 );
 
   $writer->startTag("nmwg:store",
                         "xmlns:nmtm"      =>      "http://ggf.org/ns/nmwg/time/2.0/",   
                         "xmlns:nmwgtopo3" =>      "http://ggf.org/ns/nmwg/topology/base/3.0/",	  
			 "xmlns:nmtl2" => "http://ggf.org/ns/nmwg/topology/l2/3.0/", 
                         "xmlns:nmtl3" => "http://ggf.org/ns/nmwg/topology/l3/3.0/", 
                         "xmlns:nmwg"    =>      "http://ggf.org/ns/nmwg/base/2.0/",
                         );
  
    $writer->startTag("nmwg:parameters", "id" => "storeId");
       $writer->startTag("nmwg:parameter", "name" => "DomainName");
                  $writer->characters($GENERAL_CONFIG{MYDOMAIN});
       $writer->endTag("nmwg:parameter");
  $writer->endTag("nmwg:parameters"); 
  #### storing nodes
  foreach my $datum (keys %{$nodes}) {     
     next unless $nodes->{$datum}{node_id};
      $writer->startTag("nmwg:metadata",  "id" => $datum);
         $writer->startTag("nmwg:subject",   "id" => $nodes->{$datum}{subj_id});
            $writer->startTag("nmwgtopo3:node",   "id" => $nodes->{$datum}{node_id} );
	        $writer->startTag("nmwgtopo3:type"); 
		    $writer->characters( $nodes->{$datum}{type});
	       $writer->endTag("nmwgtopo3:type");
	       $writer->startTag("nmwgtopo3:name", "type" => "logical"); 
	            $writer->characters( $nodes->{$datum}{$nodes->{$datum}{node_id}});
	       $writer->endTag("nmwgtopo3:name");
	       
               foreach my $info (qw/country city institution latitude longitude/) {
                   $writer->startTag("nmwgtopo3:$info");
                       $writer->characters($nodes->{$datum}{ $info } ) ;  
		   $writer->endTag("nmwgtopo3:$info");
	       }					
	   $writer->endTag("nmwgtopo3:node");
        $writer->endTag("nmwg:subject");											  
    $writer->endTag("nmwg:metadata"); 		
  }	   
  #### storing links 
  foreach my $datum (keys %{$config}) {     
      next unless $config->{$datum}{globalname};
      $writer->startTag("nmwg:metadata",  "id" => $datum);
         $writer->startTag("nmwg:subject",   "id" => $config->{$datum}{subj_id});
            $writer->startTag("nmtl2:link");
               $writer->startTag("nmtl2:globalName", "type" => "logical");
	          $writer->characters( $config->{$datum}{globalname} ) ;
	       $writer->endTag( "nmtl2:globalName");
	       $writer->startTag("nmtl2:type"); 
	         $writer->characters( $config->{$datum}{link_type});
	       $writer->endTag( "nmtl2:type");
               foreach my $node (keys %{$config->{$datum}{link}}) {
                   $writer->startTag("nmwgtopo3:node", "nodeIdRef" => $node);
                       $writer->startTag("nmwgtopo3:role");
		          $writer->characters( $config->{$datum}{link}->{$node});
		       $writer->endTag("nmwgtopo3:role");
		   $writer->endTag("nmwgtopo3:node");
	       }
	    $writer->endTag("nmtl2:link");
        $writer->endTag("nmwg:subject");
      $writer->startTag("nmwg:parameters");
      $writer->startTag("nmwg:parameter", "name" => "supportedEventType");
      $writer->characters('Path.Status');
      $writer->endTag("nmwg:parameter");
      $writer->endTag("nmwg:parameters");
      $writer->endTag("nmwg:eventType");											  
      $writer->endTag("nmwg:metadata"); 	
      # ---------------- data section
      $writer->startTag("nmwg:data",  "metadataIdref" => $datum);
        $writer->startTag("nmwg:key");
          $writer->startTag("nmwg:parameters");
               $writer->startTag("nmwg:parameter", "name" => "pathId");
	         $writer->characters( $config->{$datum}{globalname});
               $writer->endTag("nmwg:parameter");
	       $writer->startTag("nmwg:parameter", "name" => "ibatisConfig");
	         $writer->characters( $ibatis_configs{$config->{$datum}{link_type}});
               $writer->endTag("nmwg:parameter");
	      
         $writer->endTag("nmwg:parameters");
       $writer->endtTag("nmwg:key"); 
     $writer->endTag("nmwg:data" ); 	
  }
  $writer->endTag("nmwg:store");	 
  $fd->close; 
  move($tmp_file,  $GENERAL_CONFIG{sqlMa_config})  unless $debug;
}
#
#  store XML template for SQL MA data store request ( java based, set configuration properly, see E2EConfig.pm)
#
#
sub storeMA {
   my ($ssid, $nodes,$extnodes,  $config) = @_;

   my $tmp_file = "$GENERAL_CONFIG{tFileMA}."  . $ssid ;
   my $fd = new IO::File(">$tmp_file")  or croak("Failed to open file $tmp_file" . $!);
   
   my $namespace  = "http://ggf.org/ns/nmwg/base/2.0/";##
                                           
   my $writer = new XML::Writer(NAMESPACES => 1,
                                PREFIX_MAP => { $namespace => "message",},
                                UNSAFE => 1, OUTPUT =>  $fd, DATA_MODE => 1, DATA_INDENT => 2 );
 
   $writer->startTag("nmwg:message", 
                         "type"          =>      "MeasurementArchiveStoreRequest",
                         "xmlns:nmwg"    =>      "http://ggf.org/ns/nmwg/base/2.0/",
                         "xmlns:nmtl2" => "http://ggf.org/ns/nmwg/topology/l2/3.0/", 
                         "xmlns:select"  => "http://ggf.org/ns/nmwg/ops/select/2.0/",
                         "xmlns:ifevt" => "http://ggf.org/ns/nmwg/event/status/base/2.0/");
  
  
  #### storing links 
  foreach my $datum (keys %{$config}) {     
      next unless $config->{$datum}{globalname};
      $writer->startTag("nmwg:metadata",  "id" => $datum);
         $writer->startTag("nmwg:subject",   "id" => $config->{$datum}{subj_id});
            $writer->startTag("nmtl2:link");
               $writer->startTag("nmtl2:globalName", "type" => "logical");
	          $writer->characters( $config->{$datum}{globalname} ) ;
	       $writer->endTag( "nmtl2:globalName");
	    $writer->endTag("nmtl2:link");
        $writer->endTag("nmwg:subject");
      $writer->startTag("nmwg:eventType");
      $writer->characters('Path.Status');
       $writer->endTag("nmwg:eventType");											  
    $writer->endTag("nmwg:metadata"); 		
  }		 
   $fd->close; 
  move($tmp_file, $GENERAL_CONFIG{tFileMA})  unless $debug;
}
#
#  store XML template for E2E MP ( will be used by vlanmon.pl - E2E status polling script
#
#
sub storeTemplate {
   my ($ssid, $nodes,$extnodes,  $config) = @_;

   my $tmp_file = "$GENERAL_CONFIG{tFile}."  . $ssid ;
   my $fd = new IO::File(">$tmp_file")  or croak("Failed to open file $tmp_file" . $!);
   
   my $namespace  = "http://ggf.org/ns/nmwg/base/2.0/";
                                           
   my $writer = new XML::Writer(NAMESPACES => 1,
                                PREFIX_MAP => { $namespace => "message",
                                  },
                             UNSAFE => 1, OUTPUT =>  $fd, DATA_MODE => 1, DATA_INDENT => 2 );
 
   $writer->startTag("nmwg:message", "type" => "E2E_Link_status_information",
                         "xmlns:nmwg"       =>      "http://ggf.org/ns/nmwg/base/2.0/",
                         "xmlns:nmtm"      =>      "http://ggf.org/ns/nmwg/time/2.0/",   
                         "xmlns:nmwgtopo3" =>      "http://ggf.org/ns/nmwg/topology/base/3.0/",	  
			 "xmlns:nmtl2" => "http://ggf.org/ns/nmwg/topology/l2/3.0/", 
                         "xmlns:nmtl3" => "http://ggf.org/ns/nmwg/topology/l3/3.0/",
                         "xmlns:nmtl4" => "http://ggf.org/ns/nmwg/topology/l4/3.0/",
                         "xmlns:ifevt" => "http://ggf.org/ns/nmwg/event/status/base/2.0/");
   $writer->startTag("nmwg:parameters", "id" => "storeId");
       $writer->startTag("nmwg:parameter", "name" => "DomainName");
                  $writer->characters($GENERAL_CONFIG{MYDOMAIN});
       $writer->endTag("nmwg:parameter");
  $writer->endTag("nmwg:parameters");    
  #### storing nodes
  foreach my $datum (keys %{$nodes}) {     
     next unless $nodes->{$datum}{node_id};
      $writer->startTag("nmwg:metadata",  "id" => $datum);
         $writer->startTag("nmwg:subject",   "id" => $nodes->{$datum}{subj_id});
            $writer->startTag("nmwgtopo3:node",   "id" => $nodes->{$datum}{node_id} );
	        $writer->startTag("nmwgtopo3:type"); 
		    $writer->characters( $nodes->{$datum}{type});
	       $writer->endTag("nmwgtopo3:type");
	       $writer->startTag("nmwgtopo3:name", "type" => "logical"); 
	            $writer->characters( $nodes->{$datum}{$nodes->{$datum}{node_id}});
	       $writer->endTag("nmwgtopo3:name");
	       
               foreach my $info (qw/country city institution latitude longitude/) {
                   $writer->startTag("nmwgtopo3:$info");
                       $writer->characters($nodes->{$datum}{ $info } ) ;  
		   $writer->endTag("nmwgtopo3:$info");
	       }					
	   $writer->endTag("nmwgtopo3:node");
        $writer->endTag("nmwg:subject");											  
    $writer->endTag("nmwg:metadata"); 		
  }	
  
  #### storing links 
  foreach my $datum (keys %{$config}) {     
      next unless $config->{$datum}{globalname};
      $writer->startTag("nmwg:metadata",  "id" => $datum);
         $writer->startTag("nmwg:subject",   "id" => $config->{$datum}{subj_id});
            $writer->startTag("nmtl2:link");
               $writer->startTag("nmtl2:name", "type" => "logical");
	          $writer->characters( $config->{$datum}{$config->{$datum}{globalname}} );
	       $writer->endTag( "nmtl2:name");
	       $writer->startTag("nmtl2:globalName", "type" => "logical");
	          $writer->characters( $config->{$datum}{globalname} ) ;
	       $writer->endTag( "nmtl2:globalName");
	       $writer->startTag("nmtl2:type"); 
	         $writer->characters( $config->{$datum}{link_type});
	       $writer->endTag( "nmtl2:type");
               foreach my $node (keys %{$config->{$datum}{link}}) {
                   $writer->startTag("nmwgtopo3:node", "nodeIdRef" => $node);
                       $writer->startTag("nmwgtopo3:role");
		          $writer->characters( $config->{$datum}{link}->{$node});
		       $writer->endTag("nmwgtopo3:role");
		   $writer->endTag("nmwgtopo3:node");
	       }					
	   $writer->endTag("nmtl2:link");
        $writer->endTag("nmwg:subject");											  
    $writer->endTag("nmwg:metadata"); 		
  }		 
   $fd->close; 
   move($tmp_file, $GENERAL_CONFIG{tFile}) unless $debug;
}
#
#    parse through the E2E Mp Template XML, find datum, nodes, links...
#    return: ref to hash of links  {globallink name}{datum id:md[0-9]+}{link_type}=  name:DOMAIN_Link|ID_LinkPartialInfo or
#                                 {globallink name}{datum id:md[0-9]+}{link}  = ref to hash with link cinfig (nodeId,Node type(EndPoint|DemarcPoint))
#             and ref of hash of nodes {nodeIdRef}({type}=?,{logicalname}=?,{country}=?,{city}=?,{institution}=?,{latitude}=?,{longitude}=?)
#
sub fillConfig {
 my $i = 0;
 open INF, "<$GENERAL_CONFIG{tFile}" or croak("Failed to open file  $GENERAL_CONFIG{tFile}" . $!);
 my $xml_config =  join " ", <INF>;
 close INF;
 $xml_config  .=   '</nmwg:message>';
 my $parser = XML::XPath->new($xml_config );
 $parser->clear_namespaces();
 $parser->set_namespace('nmwg', 'http://ggf.org/ns/nmwg/base/2.0/');
 $parser->set_namespace('nmtm', 'http://ggf.org/ns/nmwg/time/2.0/' );
 $parser->set_namespace('nmwgtopo3', 'http://ggf.org/ns/nmwg/topology/base/3.0/');
 $parser->set_namespace('ifevt', 'http://ggf.org/ns/nmwg/event/status/base/2.0/');
 $parser->set_namespace('nmtl2', 'http://ggf.org/ns/nmwg/topology/l2/3.0/');  
 $parser->set_namespace('nmtl3', 'http://ggf.org/ns/nmwg/topology/l3/3.0/');
 $parser->set_namespace('nmtl4', 'http://ggf.org/ns/nmwg/topology/l4/3.0/');
 my %nlinks = (); ##  hash with all defined links ( keyed by meta data id "mdXX" )
 my %nodes = (); ####  hash with all domain nodes
 my %extnodes = (); ####  hash with all external nodes ( where XXXX is not the local domain name in XXXX-YYYYY )
 
 my  $domain_el =    $parser->find('/nmwg:message/nmwg:parameters');
 my    $MYDOMAIN =   $domain_el->string_value;
      $MYDOMAIN =~ s/\s+?//g;
 unless($MYDOMAIN &&  $GENERAL_CONFIG{MYDOMAIN} eq $MYDOMAIN) {
    croak("The document is invalid, storeId  is missed in  /nmwg:message/nmwg:parameters or there was different DOMAIN defined:$GENERAL_CONFIG{MYDOMAIN} ");
 }
 
 my $items = $parser->find('/nmwg:message/nmwg:metadata');  
    
 foreach my $item ($items->get_nodelist){ 
     
      my $id =  get_attr($item->getAttributes(), 'id');
      croak("The document is invalid, id atttribute is missed in metadata element") unless $id;
      foreach my $els ( $item->getChildNodes()) { 
	  if( $els->getLocalName && $els->getLocalName eq  'subject') { 
		#### get subject id attribute
              my $sub_id = get_attr($els->getAttributes(), 'id'); ## subject id

	      foreach my $ell ( $els->getChildNodes()) { 

	        my %linkNodes = ();
	        my $link_type = '';
	        my $globalname = ''; 
		my $logical_name = '';       
		if( $ell->getLocalName &&  $ell->getLocalName eq  'link') { 
		   foreach my $el ( $ell->getChildNodes()) { 
		     my $el_name = $el->getLocalName;
		     next unless $el_name;
		     if( $el_name  eq 'name' ) {
			 my $logical = get_attr($el->getAttributes(), 'type');
			 $logical_name = $el->string_value if($logical eq "logical"); ## get logical name
			 $logical_name =~ s/\s+?//g;
	             } elsif( $el_name  eq 'globalName' ) {
		       $globalname = $el->string_value;
		       $globalname =~ s/\s+?//g;
		     } elsif($el_name  eq 'type') {  
			 $link_type = $el->string_value;
			 $link_type =~ s/\s+?//g;
	 	     } elsif($el_name   eq 'node') {
		        
			 my $idref = get_attr($el->getAttributes(),'nodeIdRef');
			 my ($domain_part)  = $idref =~ /^(\w+)\-/;
			 if($domain_part ne $MYDOMAIN) {
			     $extnodes{$idref}++;
			 }
			  
			 $linkNodes{$idref} = $el->string_value; ##   node types ( EndPoint, DemarcPoint)
			 $linkNodes{$idref} =~ s/\s+?//g;
		     }
                  } 
		  if($id && $globalname   &&  $sub_id   && $link_type) {
		        $nlinks{$id}{$globalname} = $logical_name;
			$nlinks{$id}{globalname} = $globalname ;
			
	                $nlinks{$id}{link} = \%linkNodes; 
	                $nlinks{$id}{link_type} = $link_type;
			$nlinks{$id}{subj_id} =  $sub_id; 
	          }  
	        } elsif( $ell->getLocalName &&  $ell->getLocalName eq  'node') {

		   my $node_id = get_attr($ell->getAttributes(), 'id');   ### get node Id - this is  id   attribute in the  node element
		   my ($domain_part)  = $node_id   =~ /^(\w+)\-/;
		   if($domain_part ne $MYDOMAIN) {
			 croak("The document is invalid, XML config has configured node $node_id ($id) outside of domain $MYDOMAIN");    
		    }
		   $nodes{$id}{subj_id} = $sub_id;
		   $nodes{$id}{node_id} = $node_id;  
		   foreach my $el ( $ell->getChildNodes()) { 
		      my $node_subname = $el->getLocalName; 
		      next unless $node_subname;
	              if(   $node_subname  eq 'name') { 
			my $logical = get_attr($el->getAttributes(), 'type');
			$nodes{$id}{$node_id}   = $el->string_value  if($logical eq "logical"); ## get logical name
		      } elsif( $node_subname   =~ /^(type|country|city|latitude|institution|longitude)$/) {
			 $nodes{$id}{$node_subname}   = $el->string_value;
			 $nodes{$id}{$node_subname}   =~ s/\s+?//g; 
		      }
		  }   
	       }

	  }
      }
   }  
 } 
  
 return \%nlinks, \%nodes, \%extnodes;
} 
 
 
#
#
#   generate  XML  MP response
#

sub fillXML {
   my ($links, $what) = @_;
   my $ctime = strftime "%Y-%m-%dT%H:%M:%S.0-6:00", gmtime(time());
   my $timeformat= 'ISO';
   if($what eq 'MA') {
     $ctime =  time();
     $timeformat= 'unix';
   }
   my $oper = 'NormalOperation';
   
   my $xmls = '';
   
   my %datums = ();
   foreach my $ifid (keys %{$links}) {
   
     foreach my $ifport (keys %{$links->{$ifid}}) {
       next if $ifport eq 'rtr';  
       my $status = $links->{$ifid}->{$ifport}->{'status'};
       foreach my $data (keys %{$links->{$ifid}->{$ifport}->{'datum'}}) { 
          print "Finding  Datum= $data  State = $status " . $links->{$ifid}->{$ifport}->{'datum'}->{$data} . " \n" if $debug;
          if(   $status ==0  &&  !(exists $datums{$data}  && $datums{$data} > 0)) {
	     $datums{$data} = 0;
	  }  else {
	     $datums{$data} = 1;
	  }
      
        }
     } 
    } 
     my $d = 1;
    foreach my $data (keys %datums) { 
     
     print " Datum= $data  State = $datums{$data} \n" if $debug;
     if(  $datums{$data} == 0 ) {
         $xmls .= qq(
         <nmwg:data id="d$d" metadataIdRef="$data">
	    <ifevt:datum timeType="$timeformat" timeValue="$ctime">
	       <ifevt:stateOper>Up</ifevt:stateOper>
	      <ifevt:stateAdmin>$oper</ifevt:stateAdmin>
	    </ifevt:datum>
        </nmwg:data>
           ); 
     } elsif( $datums{$data}  == 1) {
      $xmls .= qq(
         <nmwg:data id="d$d" metadataIdRef="$data">
	    <ifevt:datum timeType="$timeformat" timeValue="$ctime">
	       <ifevt:stateOper>Down</ifevt:stateOper>
	      <ifevt:stateAdmin>$oper</ifevt:stateAdmin>
	    </ifevt:datum>
        </nmwg:data>
           ); 
    
     }  else {
      $xmls .= qq(
         <nmwg:data id="d$d" metadataIdRef="$data">
	    <ifevt:datum timeType="$timeformat" timeValue="$ctime">
	       <ifevt:stateOper>Unknown</ifevt:stateOper>
	      <ifevt:stateAdmin>$oper</ifevt:stateAdmin>
	    </ifevt:datum>
        </nmwg:data>
           ); 
    
     } 
   $d++;
  } 
  return $xmls;
} 
1;
