#!/usr/local/bin/perl -w
#
#  
#   Vlan monitor for  E2E circuits monitoring ( CISCO only!) 
#    Supposed to be ran as a cron job with redirection of STDERR/STDOUT into some logfile
#
#   author Maxim Grigoriev
#    2006
#
#   Monitors VLAN status by checking operational status of the interface, bgp session and VLAN state
#
# Some inspiration is taken from   bgpmonitor.pl by
# Copyright 2002, Marc Hauswirth, Safe Host SA <marc@safehostnet.com>
# Copyright 2005, Val Glinskiy, <vglinskiy@gmail.com>
# Some inspiration is taken from Marc Hauswirth's script http://www.kernel.org/pub/software/admin/mon/contrib/monitors/bgp/bgp.monitor
#
# License: GNU GPL v2, see http://www.gnu.org/copyleft/gpl.html
#
# This script was tested with Cisco  swithces and routers
#
#
# This script needs the Net::SNMP  module
#
 
use strict;
use lib ('/home/netadmin/LHCOPN/G2_E2E/scripts');
use Net::SNMP qw(:snmp);
use Getopt::Std;
use Carp;
use POSIX qw(strftime); 
use File::Copy;
use Pod::Usage; 
use HTML::Template; 

#######################  SNMP data structure defined here
use SNMPmonConfig qw(:Monitor);
#######################   Configuration details defined here
use E2EConfig qw(:Utils %GENERAL_CONFIG $debug);
###################### XML related scripts and config
use XMLUtils qw(  &fillXML &sendMARequest);
 
#######
## -- Nothing to configure here -------------------------------
######

$ENV{PATH} = '/usr/bin:/usr/local/bin:/bin'; 
my %opts  = ();

my %links = $SNMP_CONFIG_OBJ->getall;

my @noc_problems =  ();  

&getopts("hdc:",\%opts) or pod2usage(2); 
pod2usage(-verbose => 3)  if($opts{h}); 
if($opts{d}) {
    $debug =$opts{d}; 
} 
 
if($opts{c}) {
      $GENERAL_CONFIG{COMMUNITY} =$opts{c}; 
}  
 
## --   check if we are writing status into the status file
open OUTF , ">$GENERAL_CONFIG{statusFile}" or   croak(" !!! Failed to open $GENERAL_CONFIG{statusFile}");

 
croak("Was unable to open template  XML file:$GENERAL_CONFIG{tFile} ")  if  !$GENERAL_CONFIG{tFile} || !open INF, "<$GENERAL_CONFIG{tFile}";   
my @xmlT  = <INF>;
close INF;
 
 	
my %state = ();
my $now_gmt =   strftime "%Y-%m-%dT%H:%M:%S.0-6:00", gmtime(time());
 
my %routers = ();
my %sess = ();
#
# query the switch/router
#
foreach my $ifs (keys %links) {
    my $error = '';
    my $rtr = $links{$ifs}->{rtr};
   ($sess{$rtr}, $error) = Net::SNMP->session( -hostname => $rtr , -community =>  $GENERAL_CONFIG{COMMUNITY}, -version => '2c' ) || 
                   croak( "!!! Can not connect to $rtr !!! ") 
    unless $sess{$rtr};
    croak(" !!! ERROR $error while connecting to $rtr !!!") if $error;
}
#### check every interface IP and gather status info 
foreach my $ifs ( keys %links) {
   foreach my $key (keys %{$links{$ifs}}) {
      next if $key eq 'rtr';
      my $current_glinks = (join " <br> ",  values %{$links{$ifs}->{$key}->{'datum'}});
      
      my $rtr = $links{$ifs}->{rtr};
      my $oidIfStatus=$oids{ifOperStatus}.".".   $key ;
      my $IfState = $sess{$rtr}->get_request(-varbindlist => [$oidIfStatus]);

      if (!defined($IfState)){
          carp(" $oidIfStatus  No results\n ERROR: " . $sess{$rtr}->error);
          next;
      }
      ###### Checking physical interface
      if($IfState->{$oidIfStatus} == 1) {# 1 - Up / 2 - Down
         $links{$ifs}->{$key}->{'status'}--; 
      } else {
         push @noc_problems, {datetime => $now_gmt, 
	                      problem => "L2device: $ifs Interface:$key  is <b>DOWN</b>", 
			      globalname => $current_glinks };
	 
      }
      print " $ifs -- $key state= $IfState->{$oidIfStatus}  OP_Status= " . $links{$ifs}->{$key}->{'status'} ." \n" if $debug;
      
     ###checking BGP neighbor
     
     if($links{$ifs}->{$key}->{bgp}) { 
         my $oidPeerStatus=$oids{bgpPeerState}.".". $links{$ifs}->{$key}->{bgp};
	 my $PeerState = $sess{$rtr}->get_request(-varbindlist => [$oidPeerStatus]);

	 if (!defined($PeerState)){
             carp( "$oidPeerStatus  No results\n ERROR:" . $sess{$rtr}->error);
             next;
	 } 
	 my  $oidTime = $oids{bgpPeerFsmEstablishedTime}."."  .$links{$ifs}->{$key}->{bgp} ;
	 my $time = $sess{$rtr}->get_request(-varbindlist => [$oidTime]);
         my $dnsname = &dnsName($links{$ifs}->{$key}->{bgp});
	 print  "Neighbor:  " . $links{$ifs}->{$key}->{bgp} . "  \t$dnsname \t Status:\t",$BgpPeerState{$PeerState->{$oidPeerStatus} },
	      " TimeUp:\t ",   sectotime($time->{$oidTime}),"\n" ;

	 if($PeerState->{$oidPeerStatus} == 6 && $time->{$oidTime} > 1) {

	      $links{$ifs}->{$key}->{status}--;
	 } else {
	       push @noc_problems, {datetime => $now_gmt, 
	                            problem => "BGP status for Neighbor: IP=" . $links{$ifs}->{$key}->{bgp}  . " <i>$dnsname</i>  is <b>" .
				    $BgpPeerState{$PeerState->{$oidPeerStatus}} . "</b> <i> BUT must be <u>ESTABLISHED</u> </i>", 
			            globalname => $current_glinks };
	 }
	 print   $links{$ifs}->{$key},"   OP_Status= " .  $links{$ifs}->{$key}->{status} ." \n" if $debug;
      }
      
      ###checking vlan status
      
      if($links{$ifs}->{$key}->{vlan}) {  
	 my $oidVlanStatus = $oids{vtpVlanState} . "." . $links{$ifs}->{$key}->{vlan};
         my $VlanState = $sess{$rtr}->get_request(-varbindlist => [$oidVlanStatus]);
         if (!defined($VlanState)){
             carp(  "$oidVlanStatus  No results\n ERROR:" . $sess{$rtr}->error);
             next;
	 } 
         if ($VlanState->{$oidVlanStatus}  == 1) {# 1 - Up / 2 - Down
	       $links{$ifs}->{$key}->{status}-- ;
	 } else {
	       push @noc_problems, {datetime => $now_gmt, 
	                            problem => " VLAN State for VLAN" . $links{$ifs}->{$key}->{vlan} ."  -  is <b>DOWN</b><br>", 
			            globalname => $current_glinks };
	    
         }
	 print "VLAN  $oidVlanStatus  State= " .  $VlanState->{$oidVlanStatus} . " Status=" .  $links{$ifs}->{$key}->{status}  ." \n" if $debug;
	     
      }  

     }
}
###closing up SNMP sessions
map{$sess{$_}->close}  keys %sess; 
 
#######  generating current status report as E2E MP XML response file 
my $datums= fillXML(\%links,'MP');
print OUTF "  ",(join "\n", @xmlT), $datums,  "</nmwg:message>\n";
close OUTF;
move($GENERAL_CONFIG{statusFile}, $GENERAL_CONFIG{RealStatusFile} ); ## move used to provide atomicity
#############
##################  creating web report if requested (empty page if nothing to report)####################
if($GENERAL_CONFIG{GET_NOC_REPORT}) {
   my $TEMPLATE_LOCAL_HEADER  = HTML::Template->new(filename =>  $GENERAL_CONFIG{LOCAL_HEADER_TMPL} );
   my $TEMPLATE_NOC_REPORT  = HTML::Template->new(filename =>   $GENERAL_CONFIG{NOC_REPORT_TMPL} );
   open OUTW, ">$GENERAL_CONFIG{webpage_file}" or carp("Failed to open $GENERAL_CONFIG{webpage_file} for writing "); 
   $TEMPLATE_LOCAL_HEADER->param(title => "$GENERAL_CONFIG{MYDOMAIN} E2E monitoring Status Information - NOC Report");
   $TEMPLATE_NOC_REPORT->param(problems => \@noc_problems);
   print OUTW $TEMPLATE_LOCAL_HEADER->output(); 
   print OUTW $TEMPLATE_NOC_REPORT->output(); 
   close OUTW;
   ######  adding up to historical reports archive
   if(@noc_problems) {
      my $old_report = '';
      if( -e $GENERAL_CONFIG{webpage_noc_history})  {
         open INW, "<$GENERAL_CONFIG{webpage_noc_history}" or carp("Failed to open $GENERAL_CONFIG{webpage_noc_history} for reading "); 
	 my $start_here = undef;
	 while(<INW>) {
	    chomp;
	    if(m/^\<\!\-\-\ problem start here\ \-\-\>/) {
	         $start_here = 1;
	     } elsif(m/^\<\!\-\-\ problem end here\ \-\-\>/) {
	        $start_here = undef;
	     } elsif($start_here) {
	        $old_report .= $_ . "\n";
	     }
	 }
	 close INW;
      }
      my $TEMPLATE_NOC_HISTORY = HTML::Template->new(filename =>   $GENERAL_CONFIG{NOC_HISTORY_TMPL} );
      $TEMPLATE_NOC_HISTORY->param(old_report => $old_report);  
      $TEMPLATE_NOC_HISTORY->param(problems => \@noc_problems);
      open OUTW, "+>$GENERAL_CONFIG{webpage_noc_history}" or carp("Failed to open $GENERAL_CONFIG{webpage_noc_history} for appending "); 
      print OUTW  $TEMPLATE_LOCAL_HEADER->output(); 
      print OUTW  $TEMPLATE_NOC_HISTORY->output();  
      close OUTW;  
   }
}
#######  generate current status report as E2E MA XML data request file 
if($GENERAL_CONFIG{GET_MA_REQUEST})  {                                                                                    
    $datums= fillXML(\%links,'MA');
    croak("Was unable to open template  XML file:$GENERAL_CONFIG{tFileMA} ")  if  !$GENERAL_CONFIG{tFileMA} || !open INF, "<$GENERAL_CONFIG{tFileMA}";  
    my @xmlTMA  = <INF>;
    close INF;
    open OUTF , ">$GENERAL_CONFIG{statusFileMA}" or   croak(" !!! Failed to open $GENERAL_CONFIG{statusFileMA}");
    print OUTF "  ",(join "\n", @xmlTMA), $datums,  "</nmwg:message>\n";
    close OUTF; 
    #########   
    sendMARequest(); 
}
exit(0);


__END__


=head1 NAME

B<vlanmon.pl> -  gather current status from the CISCO devices

=head1 SYNOPSIS

B<vlanmon.pl>     B<[-h]>     B<[-d]>    B<[-c]>  [router_name]

=head1 DESCRIPTION

  this script should be run as a cron job in order to gather current status of particular circuit
  the status report based on aggregation of the physical interface status, BGP session status and
  VLAN vtp operational state. The result is compiled and published as XML file ready to be sent
  by local E2E perfSONAR Measurement Point and/or submitted to local E2E perfSONAR SQL Measurement Archive
  Configuration parameters for this script are defined by E2E Configuration management webinterface
  Or in ../etc/GeneralSystem.conf file and ../etc/SNMPmon.conf file
     
=head1 OPTIONS

=over  

 	    
=item   B<-h> 
 
          print usage info
	  
=item   B<-d>
   
        print a lot of debugging info  
	
=item   B<-c>
     
        SNMP community string ( current SNMP protocol is 2c )
	Default: public
 
=back


 
=cut
